function [outercells_all_stats, whole_heart_stats] =  zebra_angle_r1(base_path, image_dir, verbose_cell_by_cell, computer_ID)
%% Stephen Gilbert
%Zebra fish heart cell orientation and shape quantification
%This code takes the output from Packing Analyzer v 2 (Benoit Aigouy --->  Aigouy B, Farhadifar R, Staple DB, Sagner A, Röper JC, Jülicher F, Eaton S. Cell. 2010 Sep 3;142(5):773-86) and
%extracts statistics relevant to a simple local tissue "coordinate scheme" 

%%%%%%%%%%%TESTING%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% clear
% I_path = 'C:\Users\stephen\Desktop\MDC_des\zebra\Vertices 54hpf - GFP\Vertices 54hpf - GFP\MAX_C3-Vertix GFP 07A\';
% image_dir = 'MAX_C3-Vertix GFP 7A';

%I_path = [base_path, image_dir];
%%%%%%%%%%%TESTING%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I_path = [base_path, image_dir];
%load the original image
fprintf('Processing: %s \n', image_dir);


prompt = sprintf('Please input the pixel dimension in micrometers):');
pixel_DIM = input(prompt);



I_OMW = imread(strcat(I_path, 'original_made_white.png'));
XMAX = size(I_OMW, 1);
YMAX = size(I_OMW, 2);



figure, f_m1 = imshow(I_OMW);


%find edge from segmentation
Id = imread(strcat(I_path, 'cell_identity.png'));

Id_blue = Id(:,:,3);
Id_blue_BW = Id_blue;
Id_blue_BW_new = Id_blue;

indices = find(abs(Id_blue_BW)<1.1);
Id_blue_BW(indices) = 0;
indices = find(abs(Id_blue_BW)>0.9);
Id_blue_BW(indices) = 255;



raw_PC = imread(strcat(I_path, 'raw_polygon_classes.png'));
I_TCR = imread(strcat(I_path, 'border_cells.png'));
I_TCR_red =  I_TCR(:,:,1);
indices_background = find(I_TCR_red == 255); 


Id_blue_BW_new(indices_background) = 0;

Id_blue_BW_new(1,:) = 0;
Id_blue_BW_new(512,:) = 0;
Id_blue_BW_new(:,1) = 0;
Id_blue_BW_new(:,512) = 0;

I_complete_mask_BW = Id_blue_BW + raw_PC(:,:,3);
I_edge = edge(I_complete_mask_BW);



I_edge_nb = uint8(I_edge);
I_edge_nb(1,:) = 0;
I_edge_nb(512,:) = 0;
I_edge_nb(:,1) = 0;
I_edge_nb(:,512) = 0;

indices = find(I_edge_nb > 0);
I_edge_nb(indices) = 200;


I_OMW_eh(:,:,1) = I_OMW(:,:,1);
I_OMW_eh(:,:,2) = I_OMW(:,:,2);
I_OMW_eh(:,:,3) = I_OMW(:,:,3) + I_edge_nb;



fm_2 = imshow(I_OMW_eh);
%label = sprintf('IMAGE: %s', image_dir);
txt_IMAGE_DIR = text(2,530 ,image_dir, 'Color', 'blue', 'FontSize', 20);





%draw a closed polyline
fprintf('STARTING FROM the point where the lesser curvature joins the\n');
fprintf('AV -junction draw an EVENLY SPACED polyline around the ventricles\n');
fprintf('and CLOSE the polyline by selecting the first point\n');

label = {'DRAW THE POLYLINE';};
txt = text(.025,20,label, 'Color', 'red', 'FontSize', 18);


h = impoly(gca);
delete(txt);



points_poly = getPosition(h);
n_points_poly = size(points_poly, 1);
for n = 1:n_points_poly;
    points{n} = points_poly(n,:);
end;

%fit a closed smooth curves to the keypoints (using Bezier curves)


hbpoints = zebra_hobbysplines_r1(points,'debug',true,'cycle',true,'offset',[3 0]);
hbpoints(:,3) = [];
%hobbysplines(points,'debug',true,'tension',2);

axis equal;
axis off;


%find tangents
x = hbpoints(:,1);
y = hbpoints(:,2);

plot(hbpoints(:,1),hbpoints(:,2), 'magenta');



%loop around the perimeter and find the cell identities
%find true perimeter

I_complete_mask_BW_ones = I_complete_mask_BW;
indices = find(I_complete_mask_BW>1.1);
I_complete_mask_BW_ones(indices) = 1;
I_complete_mask_BW_ones_fixed = I_complete_mask_BW_ones;
I_complete_mask_BW_ones_fixed(1,:) = 0;
I_complete_mask_BW_ones_fixed(512,:) = 0;
I_complete_mask_BW_ones_fixed(:,1) = 0;
I_complete_mask_BW_ones_fixed(:,512) = 0;

I_complete_mask_BW_masked_all_out_tissue_is_1 = ones(size(I_complete_mask_BW_ones)) - double(im2uint8(I_complete_mask_BW_ones_fixed));

I_complete_mask_BW_masked_all_out_tissue_is_1_u8 = uint8(I_complete_mask_BW_masked_all_out_tissue_is_1);

Id_readytouse = Id_blue - I_complete_mask_BW_masked_all_out_tissue_is_1_u8;



Id_readytouse(1,:) = 0;
Id_readytouse(512,:) = 0;
Id_readytouse(:,1) = 0;
Id_readytouse(:,512) = 0;

%find border cells form masks 
Id_p1 = imread(strcat(I_path, 'border_cells_plus_one.png'));
Id_p1_1 = Id_p1(:,:,1);


Id_readytouse_outercells = Id_blue_BW_new;


for m=1:512;
    for n=1:512;
        if(Id_p1_1(m,n) == 255);
            Id_readytouse_outercells(m,n) = Id_blue_BW_new(m,n);
        else
            Id_readytouse_outercells(m,n) = 0;
        end;
    end;
end


I_edge(1,:) = 0;
I_edge(512,:) = 0;
I_edge(:,1) = 0;
I_edge(:,512) = 0;


I_edge(2,:) = 0;
I_edge(511,:) = 0;
I_edge(:,2) = 0;
I_edge(:,511) = 0;


[px, py] = find(abs(I_edge)>0.9);
%NOTE REVERSAL INDEXED HERE VERY IMPORTANT
points_edge = cat(2, py, px);

%pick anatomical points

%C_lc_e


fprintf('Pick the point where the lesser curvature joins the AV -junction:\n');
label = {'Pick point: LC->AV-jnct:';};
txt = text(.025,20,label, 'Color', 'cyan', 'FontSize', 18);
C_lc_e = ginput(1);
scatter(C_lc_e(:,1), C_lc_e(:,2), 'cyan');
delete(txt);


%C_lc_s
fprintf('Pick the point where the outflow joins the lesser curvature:\n');
label = {'Pick point: outflow->LC:';};
txt = text(.025,20,label, 'Color', 'cyan', 'FontSize', 18);
C_lc_s = ginput(1);
scatter(C_lc_s(:,1), C_lc_s(:,2), 'cyan');
delete(txt);

%C_gc_e
fprintf('Pick the point where the outflow joins the greater curvature:\n');
label = {'Pick point outflow->GC:';};
txt = text(.025,20,label, 'Color', 'cyan', 'FontSize', 18);
C_gc_e = ginput(1);
scatter(C_gc_e(:,1), C_gc_e(:,2), 'cyan');
delete(txt);

%C_gc_s
fprintf('Pick the point where the greater curvature joins the AV-junction:\n');
label = {'Pick point GC->AV-jnct:';};
txt = text(.025,20,label, 'Color', 'cyan', 'FontSize', 18);
C_gc_s = ginput(1);
scatter(C_gc_s(:,1), C_gc_s(:,2), 'cyan');
delete(txt);

%AN ALTERNATIVE TO CENTRE OF MASS IS TO USE A manual centroid selection

%C_cent = mean(hbpoints);
hbpoints_t = transpose(hbpoints);
%[ geom, iner, cpmo ] = polygeom(hbpoints_t(1,:), hbpoints_t(2,:));
%C_cent(:,1) = geom(2);
%C_cent(:,2) = geom(3);
%A_wh = geom(1);

[C_cent, A_wh] = polygonCentroid(hbpoints_t(1,:), hbpoints_t(2,:));
A_wh = abs(A_wh);


scatter(C_cent(1),C_cent(2), 'green');

%find additional gc points
[C_lc_e_hb_perimeter, index_Sp_C_lc_e] = zebra_nearest_point_on_perimeter_from_coord_r1(points_edge, hbpoints,  C_lc_e);
[C_lc_s_hb_perimeter, index_Sp_C_lc_s] = zebra_nearest_point_on_perimeter_from_coord_r1(points_edge, hbpoints,  C_lc_s);
[C_gc_e_hb_perimeter, index_Sp_C_gc_e] = zebra_nearest_point_on_perimeter_from_coord_r1(points_edge, hbpoints,  C_gc_e);
[C_gc_s_hb_perimeter, index_Sp_C_gc_s] = zebra_nearest_point_on_perimeter_from_coord_r1(points_edge, hbpoints,  C_gc_s);

CCW = isCounterClockwise(C_gc_e_hb_perimeter, C_gc_s_hb_perimeter, C_lc_e_hb_perimeter); %expect YES FOR A FILES, NO FOR B FILES (SAME DIRECTION AS DRAWING HERE)
if CCW==-1;    %here standard use of function is reversed as y coordinate displayed in negative order for png image
    fprintf('ANTICLOCKWISE FROM GC-outflow to GC-AV around centroid:\n');
    label = {'ANTICLOCKWISE';}; %EXPECTED B GROUP
else
    fprintf('CLOCKWISE FROM GC-outflow to GC-AV around centroid:\n');
    label = {'CLOCKWISE';}; %EXPECTED A GROUP
end;

txt_CL = text(.025,20,label, 'Color', 'red', 'FontSize', 18);


pause(0.5);

%find and add C_gc_m1 and C_gc_m2
scatter(hbpoints(index_Sp_C_gc_s:index_Sp_C_gc_e,1), hbpoints(index_Sp_C_gc_s:index_Sp_C_gc_e,2), 'cyan');
hb_len = abs(index_Sp_C_gc_e - index_Sp_C_gc_s);

if index_Sp_C_gc_e > index_Sp_C_gc_s;
    %scatter(hbpoints(index_Sp_C_gc_s:index_Sp_C_gc_e,1), hbpoints(index_Sp_C_gc_s:index_Sp_C_gc_e,2), 'cyan' )
    C_gc_m1_index = floor(index_Sp_C_gc_s + 0.33*hb_len);
    C_gc_m2_index = floor(index_Sp_C_gc_s + 0.66*hb_len);
else
    %scatter(hbpoints(index_Sp_C_gc_e:index_Sp_C_gc_s,1), hbpoints(index_Sp_C_gc_e:index_Sp_C_gc_s,2), 'cyan' )
    C_gc_m1_index = floor(index_Sp_C_gc_e + 0.33*hb_len);
    C_gc_m2_index = floor(index_Sp_C_gc_e + 0.66*hb_len);
end;

C_gc_m1 = hbpoints(C_gc_m1_index, :);
C_gc_m2 = hbpoints(C_gc_m2_index, :);

scatter(C_gc_m1(:,1), C_gc_m1(:,2), 'white');
scatter(C_gc_m2(:,1), C_gc_m2(:,2), 'green');

%segmentation vectors
V_lc_e = [(C_lc_e(:,1) - C_cent(:,1)), (C_lc_e(:,2) - C_cent(:,2))];
V_lc_s = [(C_lc_s(:,1) - C_cent(:,1)), (C_lc_s(:,2) - C_cent(:,2))];
V_gc_e = [(C_gc_e(:,1) - C_cent(:,1)), (C_gc_e(:,2) - C_cent(:,2))];
V_gc_m1 = [(C_gc_m1(:,1) - C_cent(:,1)), (C_gc_m1(:,2) - C_cent(:,2))];
V_gc_m2 = [(C_gc_m2(:,1) - C_cent(:,1)), (C_gc_m2(:,2) - C_cent(:,2))];
V_gc_s = [(C_gc_s(:,1) - C_cent(:,1)), (C_gc_s(:,2) - C_cent(:,2))];

%boundary intercepts
bounds_V_lc_e = zebra_boundary_intersections_fix_lines_exout_r1(XMAX, YMAX, C_cent, C_lc_e, 'cyan', 'green', 0);
bounds_V_lc_s = zebra_boundary_intersections_fix_lines_exout_r1(XMAX, YMAX, C_cent, C_lc_s, 'cyan', 'green', 0);
bounds_V_gc_e = zebra_boundary_intersections_fix_lines_exout_r1(XMAX, YMAX, C_cent, C_gc_e, 'cyan', 'green', 0);
bounds_V_gc_m1 = zebra_boundary_intersections_fix_lines_exout_r1(XMAX, YMAX, C_cent, C_gc_m1, 'white', 'green', 0);
bounds_V_gc_m2 = zebra_boundary_intersections_fix_lines_exout_r1(XMAX, YMAX, C_cent, C_gc_m2, 'green', 'green', 0);
bounds_V_gc_s = zebra_boundary_intersections_fix_lines_exout_r1(XMAX, YMAX, C_cent, C_gc_s, 'cyan', 'green', 0);

%examine boundary intersections and exclude distant boundaries select triangle segementation
C_bound_lc_e = zebra_find_rel_bound_r1(C_lc_e, bounds_V_lc_e);
C_bound_lc_s = zebra_find_rel_bound_r1(C_lc_s, bounds_V_lc_s);
C_bound_gc_e = zebra_find_rel_bound_r1(C_gc_e, bounds_V_gc_e);
C_bound_gc_m1 = zebra_find_rel_bound_r1(C_gc_m1, bounds_V_gc_m1);
C_bound_gc_m2 = zebra_find_rel_bound_r1(C_gc_m2, bounds_V_gc_m2);
C_bound_gc_s = zebra_find_rel_bound_r1(C_gc_s, bounds_V_gc_s);


%add perimeter points
f(1) = scatter(C_bound_lc_e(:,1), C_bound_lc_e(:,2), 'yellow'); 
f(2) = scatter(C_bound_lc_s(:,1), C_bound_lc_s(:,2), 'yellow'); 
f(3) = scatter(C_bound_gc_e(:,1), C_bound_gc_e(:,2), 'yellow');
f(6) = scatter(C_bound_gc_m1(:,1), C_bound_gc_m1(:,2), 'yellow'); 
f(5) = scatter(C_bound_gc_m2(:,1), C_bound_gc_m2(:,2), 'yellow'); 
f(6) = scatter(C_bound_gc_s(:,1), C_bound_gc_s(:,2), 'yellow'); 

%draw lines for sectors from C_cent to C_bound
f_2(1) = line([C_cent(:,1) C_bound_lc_e(:,1)], [C_cent(:,2) C_bound_lc_e(:,2)], 'Color', 'white');
f_2(2) = line([C_cent(:,1) C_bound_lc_s(:,1)], [C_cent(:,2) C_bound_lc_s(:,2)], 'Color', 'white');
f_2(3) = line([C_cent(:,1) C_bound_gc_e(:,1)], [C_cent(:,2) C_bound_gc_e(:,2)], 'Color', 'white');
f_2(4) = line([C_cent(:,1) C_bound_gc_m1(:,1)], [C_cent(:,2) C_bound_gc_m1(:,2)], 'Color', 'white');
f_2(5) = line([C_cent(:,1) C_bound_gc_m2(:,1)], [C_cent(:,2) C_bound_gc_m2(:,2)], 'Color', 'white');
f_2(6) = line([C_cent(:,1) C_bound_gc_s(:,1)], [C_cent(:,2) C_bound_gc_s(:,2)], 'Color', 'white');

%make sector arrays
S_lc(:,1) = [C_cent(:,1); C_bound_lc_e(:,1); C_bound_lc_s(:,1)];
S_lc(:,2) = [C_cent(:,2); C_bound_lc_e(:,2); C_bound_lc_s(:,2)];

S_oft(:,1) = [C_cent(:,1); C_bound_lc_s(:,1); C_bound_gc_e(:,1)];
S_oft(:,2) = [C_cent(:,2); C_bound_lc_s(:,2); C_bound_gc_e(:,2)];

S_gc_A(:,1) = [C_cent(:,1); C_bound_gc_e(:,1); C_bound_gc_m1(:,1)];
S_gc_A(:,2) = [C_cent(:,2); C_bound_gc_e(:,2); C_bound_gc_m1(:,2)];

S_gc_B(:,1) = [C_cent(:,1); C_bound_gc_m1(:,1); C_bound_gc_m2(:,1)];
S_gc_B(:,2) = [C_cent(:,2); C_bound_gc_m1(:,2); C_bound_gc_m2(:,2)];

S_gc_C(:,1) = [C_cent(:,1); C_bound_gc_m2(:,1); C_bound_gc_s(:,1)];
S_gc_C(:,2) = [C_cent(:,2); C_bound_gc_m2(:,2); C_bound_gc_s(:,2)];

S_avj(:,1) = [C_cent(:,1); C_bound_gc_s(:,1); C_bound_lc_e(:,1)];
S_avj(:,2) = [C_cent(:,2); C_bound_gc_s(:,2); C_bound_lc_e(:,2)];

f_3(1) = scatter(S_lc(:,1), S_lc(:,2), 'cyan');        %SECTOR 1
f_3(2) = scatter(S_oft(:,1), S_oft(:,2), 'cyan');          %SECTOR 2
f_3(3) = scatter(S_gc_A(:,1), S_gc_A(:,2), 'cyan');    %SECTOR 3
f_3(4) = scatter(S_gc_B(:,1), S_gc_B(:,2), 'cyan');   %SECTOR 4
f_3(5) = scatter(S_gc_C(:,1), S_gc_C(:,2), 'cyan');    %SECTOR 5
f_3(6) = scatter(S_avj(:,1), S_avj(:,2), 'cyan');        %SECTOR 6


%combine sectors into a single vector
%     S_lc            %SECTOR 1
%     S_o             %SECTOR 2
%     S_gc_A          %SECTOR 3
%     S_gc_B          %SECTOR 4
%     S_gc_C          %SECTOR 5
%     S_av            %SECTOR 6
S_all_sectors(:,1,1) = S_lc(:,1);
S_all_sectors(:,2,1) = S_lc(:,2);

S_all_sectors(:,1,2) = S_oft(:,1);
S_all_sectors(:,2,2) = S_oft(:,2);

S_all_sectors(:,1,3) = S_gc_A(:,1);
S_all_sectors(:,2,3) = S_gc_A(:,2);

S_all_sectors(:,1,4) = S_gc_B(:,1);
S_all_sectors(:,2,4) = S_gc_B(:,2);

S_all_sectors(:,1,5) = S_gc_C(:,1);
S_all_sectors(:,2,5) = S_gc_C(:,2);

S_all_sectors(:,1,6) = S_avj(:,1);
S_all_sectors(:,2,6) = S_avj(:,2);

%test in_sector
%in = inpolygon(262, 321, S_avj(:,1), S_avj(:,2));


%some testing code for zebra_whichSector_r1
% C_r_point(:,1) = floor(XMAX * rand());
% C_r_point(:,2) = floor(YMAX * rand());
% [sector_num] = zebra_whichSector_r1(C_r_point, S_all_sectors);
% scatter(C_r_point(:,1), C_r_point(:,2), 'green');

%loop around the perimeter spline and for each point allocate the 
%make a list of peripheral cells
outercells = unique(Id_readytouse_outercells);
outercells(1,:) = [];
%outercells_all_stats = cat(2, zeros(size(outercells,1),1, 'double'),  zeros(size(outercells,1),1, 'double'), zeros(size(outercells,1),1), zeros(size(outercells,1),1), zeros(size(outercells,1),1));
outercells_all_stats = zeros(size(outercells,1),20, 'double');

hold on

%find tangents
x = hbpoints(:,1);
y = hbpoints(:,2);

plot(x,y);


%for each cell find the nearest point to centroid on periphery
fprintf('\n');
for n=1:size(outercells_all_stats,1)
    fprintf('.');
    cell_image = Id_blue_BW_new;
    outercells_all_stats(n,1) = n;   %original list order
    outercells_all_stats(n,2) = outercells(n);   %cell_ID
    cell_image_indices = find(Id_readytouse_outercells ~= outercells_all_stats(n,2));
    cell_image(cell_image_indices) = 0;
    %imshow(cell_image, [0 outercells_all_stats(n,2)])
    [px, py] = find(abs(cell_image)>0.9);
    % %NOTE REVERSAL INDEXED HERE VERY IMPORTANT
    points_cell = cat(2, py, px);
    %IF CELL ARE IS NOT NEED THIS WORKS VERY WELL AND IS VERY FAST
    %mean_cell_point = mean(points_cell); 
    %make polygon
    hull = convexHull(points_cell);
    [mean_cell_point, A_cell] = polygonCentroid(hull);
    A_cell = abs(A_cell);
    drawPolygon(hull);
   
    if A_cell > 2500
        prompt = sprintf('Current cell area: "%f:, continue (1/0)): ', A_cell);
        result = input(prompt);
    end;
       

    outercells_all_stats(n,3) = mean_cell_point(1);    %cc_x
    outercells_all_stats(n,4) = mean_cell_point(2);    %cc_y
    scatter(mean_cell_point(1), mean_cell_point(2), 'green');
    
    outercells_all_stats(n, 15) = zebra_whichSector_r1(mean_cell_point, S_all_sectors);    %SECTOR ID
    [C_mcp_hb_perimeter, index_hb] = zebra_nearest_point_on_perimeter_from_coord_r1(points_edge, hbpoints,  mean_cell_point);
    
    outercells_all_stats(n,5) = C_mcp_hb_perimeter(1);  %closest_hppoint_x
    outercells_all_stats(n,6) = C_mcp_hb_perimeter(2);   %closest_hppoint_y
    outercells_all_stats(n,7) = index_hb;     %closest_hppoint_index
   
    outercells_all_stats(n,19) = A_cell;     %area from hull polygons
    outercells_all_stats(n,20) = A_cell * pixel_DIM * pixel_DIM;      %corrected area in micrometer sq. from hull polygons
    
    f_peri = scatter(C_mcp_hb_perimeter(1), C_mcp_hb_perimeter(2), 'green');
end;


%print cell numbers in order from start

%END%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%% POPULATE A LIST OF THE PERIPHERAL CELLS (NOT ORDERED %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%% CODE TO FIND THE ORDERED LIST OF PERIPHERAL CELLS %%%%%%%%%%%%%%%
%START%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ignore C_uiss, and instead the next cell in each case is defined as the
%closest cell in same sector (euclidian centre to centre) distnace to
%cell_1. If no closest cell in the same sector then closest cell in the
%next sector



%start at C_lc_e
curr_sector = 1;
[start_point, index_hb] = zebra_nearest_point_on_perimeter_from_coord_r1(points_edge, hbpoints,  C_lc_e);
last_cell_index = 0;
cell_order_counter = 1;
[curr_cell_index, curr_sector] = zebra_find_closest_cell_in_this_or_next_sector_v2_r1(outercells_all_stats, S_all_sectors, curr_sector, last_cell_index, start_point);
scatter(outercells_all_stats(curr_cell_index,3), outercells_all_stats(curr_cell_index,4), 'green');
outercells_all_stats(curr_cell_index,8) = cell_order_counter;
start_point = [0,0];
%fprintf('last_cell: %d, curr_cell: %d, curr_sector: %d, cell_order_counter: %d\n', last_cell_index, curr_cell_index, curr_sector, cell_order_counter);
    fprintf('\n');
for nr=1:(size(outercells_all_stats,1)-1);
    while curr_sector < 7; %this stops loop if any cells are outside a sector, which should not happen if polyline was well drawn
    %ginput(1)
    %fprintf('last_cell: %d, curr_cell: %d, curr_sector: %d, cell_order_counter: %d\n', last_cell_index, curr_cell_index, curr_sector, cell_order_counter);
    fprintf('.');
    last_cell_index = curr_cell_index;
    cell_order_counter = cell_order_counter + 1;
    [curr_cell_index, curr_sector] = zebra_find_closest_cell_in_this_or_next_sector_v2_r1(outercells_all_stats, S_all_sectors, curr_sector, last_cell_index, start_point);
    scatter(outercells_all_stats(curr_cell_index,3), outercells_all_stats(curr_cell_index,4), 'yellow');
    outercells_all_stats(curr_cell_index,8) = cell_order_counter;
    end;
end;

%END%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%% CODE TO FIND THE ORDERED LIST OF PERIPHERAL CELLS %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        
%populate outercells_all_stats with elongation tensor
elong_tens = importdata(strcat(I_path, 'elongation_tensor.csv'));
    fprintf('\n');
for n_outercells_all_stats_original_numbering=1:(size(outercells_all_stats,1));
    fprintf('.');
    curr_cell_ID = outercells_all_stats(n_outercells_all_stats_original_numbering,2);
    curr_index_in_elong_tens_data = find(elong_tens(:,1) == curr_cell_ID);
    outercells_all_stats(n_outercells_all_stats_original_numbering,9) = elong_tens(curr_index_in_elong_tens_data, 2); %outercells_all_stats_9 = et1
    outercells_all_stats(n_outercells_all_stats_original_numbering,10) = elong_tens(curr_index_in_elong_tens_data, 3); %outercells_all_stats_9 = et2
    outercells_all_stats(n_outercells_all_stats_original_numbering,11) = elong_tens(curr_index_in_elong_tens_data, 4); %outercells_all_stats_9 = et3
    outercells_all_stats(n_outercells_all_stats_original_numbering,12) = elong_tens(curr_index_in_elong_tens_data, 5); %outercells_all_stats_9 = et4
    outercells_all_stats(n_outercells_all_stats_original_numbering,13) = 0; %unused field
    outercells_all_stats(n_outercells_all_stats_original_numbering,17) = size(find(Id_readytouse_outercells==curr_cell_ID),1); %area in pixels
end;

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%% SHAPE AVERAGING %%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% hold on;
% %get points for a cell
% max_pts = max(outercells_all_stats(:,13));
% array_all_cell_points = zeros(max_pts, 2, (size(outercells_all_stats,1)), 'double');
% for n_outercells_all_stats_original_numbering=1:(size(outercells_all_stats,1));
%     Id_readytouse_outercells_for_curr_cell = Id_readytouse_outercells;
%     Id_readytouse_outercells_for_curr_cell(:,:) = 0;
%     curr_cell_ID = outercells_all_stats(n_outercells_all_stats_original_numbering,2);
%     indices = find(Id_readytouse_outercells == curr_cell_ID);
%     Id_readytouse_outercells_for_curr_cell(indices) = 1;
%     Id_readytouse_outercells_for_curr_cell_edge = edge(Id_readytouse_outercells_for_curr_cell);
%     %NOTE REVERSAL INDEXED HERE VERY IMPORTANT
%     [px, py] = find(Id_readytouse_outercells_for_curr_cell_edge>0.9);
%     %NOTE REVERSAL INDEXED HERE VERY IMPORTANT
%     currpoints = cat(2, py, px);
%     currpoints(:,1) = currpoints(:,1) - min(currpoints(:,1));
%     currpoints(:,2) = currpoints(:,2) - min(currpoints(:,2));
%     array_all_cell_points(1:(size(currpoints)),:, n_outercells_all_stats_original_numbering) = currpoints;
%     scatter(currpoints(:,1), currpoints(:,2), [10], [1/n_outercells_all_stats_original_numbering 1/n_outercells_all_stats_original_numbering 1/n_outercells_all_stats_original_numbering ])
% end;
% 
% 
% %[mean_shape pc_shape std_shape pc_projection new_pt_out]=tangent_pca_shape(array_all_cell_points);
% %scatter(mean_shape(:,1), mean_shape(:,2), 'red');
% hold off;
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%% SHAPE AVERAGING %%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hold on;
%get points for a cell

plot(hbpoints(:,1),hbpoints(:,2), 'blue');
max_pts = max(outercells_all_stats(:,13));
array_all_cell_points = zeros(max_pts, 2, (size(outercells_all_stats,1)), 'double');
fprintf('\n');
for n_outercells_all_stats_original_numbering=1:(size(outercells_all_stats,1));
%	
    %hold on;
    fprintf('.');
    Id_readytouse_outercells_for_curr_cell = Id_readytouse_outercells;
    Id_readytouse_outercells_for_curr_cell(:,:) = 0;
    curr_cell_ID = outercells_all_stats(n_outercells_all_stats_original_numbering,2);
    indices = find(Id_readytouse_outercells == curr_cell_ID);
    Id_readytouse_outercells_for_curr_cell(indices) = 1;
    Id_readytouse_outercells_for_curr_cell_edge = edge(Id_readytouse_outercells_for_curr_cell);
    %NOTE REVERSAL INDEXED HERE VERY IMPORTANT
    [px, py] = find(Id_readytouse_outercells_for_curr_cell_edge>0.9);
    %NOTE REVERSAL INDEXED HERE VERY IMPORTANT
    currpoints = cat(2, py, px);
    %currpoints(:,1) = currpoints(:,1) - min(currpoints(:,1));
    %currpoints(:,2) = currpoints(:,2) - min(currpoints(:,2));
    array_all_cell_points(1:(size(currpoints)),:, n_outercells_all_stats_original_numbering) = currpoints;
    
    t1 = outercells_all_stats(n_outercells_all_stats_original_numbering,9);
    t2 = outercells_all_stats(n_outercells_all_stats_original_numbering,10);
    t3 = outercells_all_stats(n_outercells_all_stats_original_numbering,11);
    t4 = outercells_all_stats(n_outercells_all_stats_original_numbering,12);
    
    %%%%%%%%%%%%eig method%%%%%%%%%%%%%%
    tw = [t3, t4; t4,-t3];
    [v d] = eig(tw);
    v_n = v(1,:);
    v_a = v(2,:);
  
    
    %select the cell vector direction based on the local heart 'coordinate
    %system'
    v1 = v_a;
    v2 = -1*v_a;
    
    
    v3 = [ 0 0]; %v3 will later be assigned to selected dirrection corrected vector of cell axis.
       
    %by definition we will use the vector moving towards the heart centroid , starting from the cell centroid    
    start(1,1) = outercells_all_stats(n_outercells_all_stats_original_numbering, 3);
    start(1,2) = outercells_all_stats(n_outercells_all_stats_original_numbering, 4);
    
    if isPointInPolygon(start, hbpoints) == 0;
        fprintf('\nERROR: the centre of this cell %d (#myID) IS NOT IN THE outline drawn for the whole heart: please reject and redraw\n', outercells_all_stats(n_outercells_all_stats_original_numbering, 1));
    end;
   
    
    end_v1 = start;
    end_v2 = start;
    
    D_start_CENTRE  = sqrt(sum((([start(:,1), start(:,2)]) - C_cent) .^ 2));
    %fprintf('Distance from centroid of cell to centroid of heart:  %f\n', D_start_CENTRE);
    D_end_v1_CENTRE = D_start_CENTRE;
    D_end_v2_CENTRE = D_start_CENTRE;
    
    min_D_end_v1_CENTRE = D_start_CENTRE;
    min_D_end_v2_CENTRE = D_start_CENTRE;

    
        
    counter = 0;
    while (abs(D_start_CENTRE) - abs(D_end_v1_CENTRE) < 3) && (abs(D_start_CENTRE) - abs(D_end_v2_CENTRE) < 3) && counter < 10;
        counter = counter + 1;
        end_v1 = [end_v1(1,1) + v1(1,1) , end_v1(1,2) + v1(1,2)];
        end_v2 = [end_v2(1,1) + v2(1,1) , end_v2(1,2) + v2(1,2)];
        D_end_v1_CENTRE  =  sqrt(sum((end_v1 - C_cent) .^ 2));
        D_end_v2_CENTRE  =  sqrt(sum((end_v2 - C_cent) .^ 2));
        
        if D_end_v1_CENTRE < min_D_end_v1_CENTRE;
            min_D_end_v1_CENTRE = D_end_v1_CENTRE;
        end;
        
        if D_end_v2_CENTRE < min_D_end_v2_CENTRE;
            min_D_end_v2_CENTRE = D_end_v2_CENTRE;
        end;
            
   end;
    
   
    minD = 0;
    if min_D_end_v1_CENTRE < D_start_CENTRE && min_D_end_v1_CENTRE < min_D_end_v2_CENTRE
        %fprintf('using v1\n');
        minD = min_D_end_v1_CENTRE;
        v3 = v1;
    elseif min_D_end_v2_CENTRE < D_start_CENTRE && min_D_end_v2_CENTRE < min_D_end_v1_CENTRE
        %fprintf('using v2\n')
        minD = min_D_end_v2_CENTRE;
        v3 = v2;
    else;
        fprintf('\n#######CELL VECTOR EVALAUATION ERROR, (cell %d [my#)) - Try redrawing the polyline, if the problem persists or is multiple PLEASE REPORT ', outercells_all_stats(n_outercells_all_stats_original_numbering, 1));
        f_cell_points3 = scatter(currpoints(:,1), currpoints(:,2), 2, 'magenta');

        fprintf('using v2\n');
        v3 = v2;
    end;
        
    gamma =  max(max(d));
    gamma_f = gamma*0.33;  %This gives plotting like in packing analyser
    %gamma_f = gamma*3.0;  %This allows easy visualisation of the ange to normal (beta)
    curr_mean_cell_point = mean(currpoints);
    
    %tangent at cell
    index = outercells_all_stats(n_outercells_all_stats_original_numbering,7);% closest hpoint

    %%%%%%%%%%%%%%
    dy=diff(y)./diff(x);
    k=index; % point number 220
    if k <= size(dy,1);
        tang=(x-x(k))*dy(k)+y(k);
    end;
    hold on;
    set(gca,'XLim',[0 512], 'YLim',[0 512]);
    p = polyfit(x,tang,1);
    
    %this calculus works unless the tangent is vertical or horizontal
    %the following is crude but functional and copes with vertical or
    %horizontal tangents
    eval_vert_horiz = 0;
    if isnan(p(1)) || isnan(p(2));
        fprintf('******** Evaluating a verticle or horizontal tangent ********\n');
        eval_vert_horiz = 1;
        if size(hbpoints,1) > 10
            if index > 5 && index < size(hbpoints,1) - 5;
                fprintf('using direct linear fit for tangent\n');
                x1(1) = x(index-4);
                y1(1) = y(index-4);
                x1(2) = x(index+4);
                y1(2) = y(index+4);          
                p = polyfit(x1,y1,1);
            elseif index < size(hbpoints,1) - 9;
                fprintf('using direct linear fit for tangent\n');
                x1(1) = x(index);
                y1(1) = y(index);
                x1(2) = x(index+8);
                y1(2) = y(index+8);          
                p = polyfit(x1,y1,1);
            elseif index > 9;
                fprintf('using direct linear fit for tangent\n');
                x1(1) = x(index-8);
                y1(1) = y(index-8);
                x1(2) = x(index);
                y1(2) = y(index);   
                p = polyfit(x1,y1,1);
            end;         
        end;
    end;
   
    %vector of tangent
    %consider 2 points
    %point 1 is the hbpoint of the cell 
    hb_x = outercells_all_stats(n_outercells_all_stats_original_numbering,5); % hb_points x
    hb_y = outercells_all_stats(n_outercells_all_stats_original_numbering,6); % hb_points y
    %point 2 is the point 
    far_x = (hb_x + 20);
    far_y = p(:,1)*(hb_x + 20) + p(:,2);
    tang_v = [(far_x - hb_x), (far_y - hb_y)];
    
    
    %phi is angle to tangent
%     phi = (acos((dot(tang_v, v2))/(norm(tang_v) * norm(v2))));
%     phi_deg = circ_rad2ang(phi);
    
    
    
    %find alpha, the angle between the tangent and the normal
    
    %THE FOLLOWING SECTION SELECT THE NORMAL TO THE HEART SURFACE THAT IS
    %POINTING INTO THE HEART.
    %n = inside_normal2d(tang_v) %this function, to select the best normal
    %is not yet implemented - just an optional idea
    n1 = normalize([-tang_v(:,2), tang_v(:,1)]);  %take one normal
    n2 = normalize([tang_v(:,2), -tang_v(:,1)]);
    
    n3 = [0 0]; %n3 will be assigned to the inwards pointing normal
        
    start(1,1) = outercells_all_stats(n_outercells_all_stats_original_numbering, 5);
    start(1,2) = outercells_all_stats(n_outercells_all_stats_original_numbering, 6);

    %advance 1 point in x along along the normal
    %end_n1 = [start(1,1) + n1(1,1) , start(1,2) + n1(1,2)] 
    isPointOnPolyline(start, hbpoints);
      
    eval_n1 = 0;
    eval_n2 = 0;

    end_n1 = start;
    counter_n1 = 0;
    %advance along n1 from the polygon surface
    while isPointOnPolyline(end_n1, hbpoints) == 1 && counter_n1 <= 10
                counter_n1 = counter_n1 + 1;
                end_n1 = [end_n1(1,1) + n1(1,1) , end_n1(1,2) + n1(1,2)];
                eval_n1 = isPointInPolygon(end_n1, hbpoints);
    end;
    
    
    
    end_n2 = start;
    counter_n2 = 0;
    while isPointOnPolyline(end_n2, hbpoints) == 1 && counter_n2 <= 10
                counter_n2 = counter_n2 + 1;
                end_n2 = [end_n2(1,1) + n2(1,1) , end_n2(1,2) + n2(1,2)];
                eval_n2 = isPointInPolygon(end_n2, hbpoints);
    end;
    
    %EVALUATE WHICH NORMAL
    if eval_n1 == 1 && eval_n2 == 0;
        %fprintf('n1 is the internal normal\n');
        n3 = n1;   
    elseif eval_n2 == 1 && eval_n1 == 0;
        %fprintf('n2 is the internal normal\n');
        n3 = n2;
    elseif eval_n1 == 1 && eval_n2 == 1 && counter_n1 < counter_n2;
        %fprintf('n1 is the internal normal\n');
        n3 = n1;
    elseif eval_n1 == 1 && eval_n2 == 1 && counter_n2 < counter_n1;
        fprintf('n1 is the internal normal\n');
        n3 = n2;
    else
        fprintf('#######NORMAL EVALAUATION ERROR - PLEASE REPORT ');
    end;
    
    
  
    %alpha is angle to the normal
    norm_f = 50;
    p_n1 = [curr_mean_cell_point(:,1)+(norm_f*n3(:,1)), curr_mean_cell_point(:,2)+(norm_f*n3(:,2))];
    p_a1 = [curr_mean_cell_point(:,1)+(norm_f*v3(:,1)), curr_mean_cell_point(:,2)+(norm_f*v3(:,2))];
    
    
    
    %will be clockwise (0) in the positive region of the polar plot 0-90
    %degrees as the y axis direction is increasing from top to bottom of
    %screen.
    angle_CW = isCounterClockwise(start, p_n1, p_a1);
    alpha = (acos((dot(n3, v3))/(norm(n3) * norm(v3))));
    alpha_deg = circ_rad2ang(alpha);

     if alpha_deg > 90
          alpha_deg = alpha_deg - 180;
          
     end;
     
     if angle_CW == 1 % negative region
        alpha_deg = alpha_deg * -1;
     end;
     

    outercells_all_stats(n_outercells_all_stats_original_numbering,14) =  alpha_deg;  %ANGLE TO TANGENT IN DEGREES
    outercells_all_stats(n_outercells_all_stats_original_numbering,16) =  gamma;  %gamma eigenvalue
    
    %add corrected direct area
    outercells_all_stats(n_outercells_all_stats_original_numbering,18) = outercells_all_stats(n_outercells_all_stats_original_numbering, 17) * pixel_DIM * pixel_DIM; %corrected area in micrometer squared

    
   if verbose_cell_by_cell == 1;
        fprintf('You are in verbose mode, click/return to move to the next cell\n');
          
        %reject box
        rej_poly = [3,437;70,437;70,509;0,509];
        drawPolygon(rej_poly, 'lineWidth',5,'Color', 'red');
        
        f_cell_points = scatter(currpoints(:,1), currpoints(:,2), 2, 'yellow');
        f_cell_axis = line([curr_mean_cell_point(:,1)-gamma_f*(50*v3(:,1)) curr_mean_cell_point(:,1)+gamma_f*(50*v3(:,1))],[curr_mean_cell_point(:,2)-gamma_f*(50*v3(:,2)) curr_mean_cell_point(:,2)+gamma_f*(50*v3(:,2))], 'LineWidth', 2, 'Color', 'red');
        %norm_f = 50.0;
        f_cell_v3 = line([curr_mean_cell_point(:,1), p_a1(:,1)],[curr_mean_cell_point(:,2), p_a1(:,2)], 'LineWidth', 3, 'Color', 'green');

        %f_tl = line([hb_x far_x],[hb_y far_y], 'Color', 'white');
        f_tl2 = plot(x,tang, 'cyan');
        f_far =  plot([hb_x far_x], [hb_y far_y], 'green');
        %plot NORMAL
        norm_f = 50.0;
       
        f_norm = line([curr_mean_cell_point(:,1), p_n1(:,1)],[curr_mean_cell_point(:,2), p_n1(:,2)], 'LineWidth', 2, 'Color', 'magenta');

     
        header_outer_cells_all_stats = {
            'my#',                              %1
            'PA_Cell_ID',                       %2
            'Centroid_X',                       %3
            'Centroid_Y',                       %4
            'Perimeter_X',                      %5
            'Perimeter_Y',                      %6
            'Perimeter_Index',                  %7
            'Perimeter_order',                  %8
            'tensor_file_1',                    %9
            'tensor_file_2',                    %10
            'tensor_file_3',                    %11
            'tensor_file_4',                    %12
            'Unused',                           %13
            'Alpha',                            %14
            'Sector',                           %15
            'Gamma',                            %16
            'Area (direct) - pixels',           %17
            'Area (direct)- micrometers',       %18
            'Area (indirect) - pixels',         %19
            'Area (indirect)- micrometers'};    %20
        
        
        %pna =polygonNormalAngle(hbpoints ,
        %outercells_all_stats(n_outercells_all_stats_original_numbering, 7));
        
        for n=1:size(outercells_all_stats,2);
            fprintf('%s: %3.4f\n', char(header_outer_cells_all_stats(n)), outercells_all_stats(n_outercells_all_stats_original_numbering, n));
        end;
    
        fprintf('click in the red box to reject a cell from analysis, or else click outside the red box to keep\n')
        acc_rej_point = ginput(1);
        %evaluate accept or reject cell
        %if inpolygon(acc_rej_point, 321, S_avj(:,1), S_avj(:,2));
        if isPointInPolygon(acc_rej_point, rej_poly) == 1;
            outercells_all_stats(n_outercells_all_stats_original_numbering,14) =  NaN;  %ANGLE TO TANGENT IN DEGREES

            fprintf('---------------------Cell Rejected---------------------\n');
            f_cell_points2 = scatter(currpoints(:,1), currpoints(:,2), 10, 'red');
            pause(1.0);

        else;
            fprintf('---------------------Cell Accepted---------------------\n');
             f_cell_points2 = scatter(currpoints(:,1), currpoints(:,2), 2, 'white');
        end;
            
    

    end;
    





        if exist('f_cell_points');
            delete(f_cell_points);
        end;
        
        if exist('f_cell_axis');
            delete(f_cell_axis);
        end;
        
        if exist('f_tl2');
            delete(f_tl2);
        end;
        
        if exist('f_far');
            delete(f_far);
        end;
        
        if exist('f_norm');
            delete(f_norm);
        end;
        
        if exist('f_cell_v3');
            delete(f_cell_v3);
        end;
        
        
    
end
    

%Determine whole heart statistics
%corrected whole area (micrometer sq.)
whole_heart_stats(1,1) = A_wh * pixel_DIM * pixel_DIM;

%find vector of mid-point AV to centroid
C_mid_AV = mean(cat(1,C_lc_e_hb_perimeter, C_gc_s_hb_perimeter));
scatter(C_mid_AV(:,1), C_mid_AV(:,2), 'red');
v_mid_AV = normalize([C_mid_AV(:,1) - C_cent(:,1), C_mid_AV(:,2) - C_cent(:,2)]);
f_740 = line([C_mid_AV(:,1), C_cent(:,1)], [C_mid_AV(:,2), C_cent(:,2)], 'Color', 'green');

%find vector of mid-point OFT to centroid
C_mid_OFT = mean(cat(1,C_lc_s_hb_perimeter, C_gc_e_hb_perimeter));
scatter(C_mid_OFT(:,1), C_mid_OFT(:,2), 'green');
v_mid_OFT = normalize([C_mid_OFT(:,1) - C_cent(:,1), C_mid_OFT(:,2) - C_cent(:,2)]);
f_748 = line([C_mid_OFT(:,1), C_cent(:,1)], [C_mid_OFT(:,2), C_cent(:,2)], 'Color', 'green');
beta = (acos((dot(v_mid_OFT, v_mid_AV))/(norm(v_mid_OFT) * norm(v_mid_AV))));
beta_deg = circ_rad2ang(beta); %note beta_deg on scale 0 - 360, but in reality should not have this range except in situs invesus totalis

if beta_deg < 90 || beta_deg > 185;
    fprintf('**************WARNING: the cardiac axis is outside the expected range of 90 - 185 degrees');
    prompt = sprintf('Continue (1/1)): ');
    result = input(prompt);
end;
    


%NOTE THE ABSOLUTE CARDIAC AXIS ANGLE IS REPORTED
whole_heart_stats(1,2) = beta_deg;

%WHOLE HEART CLOCKWISE
whole_heart_stats(1,3) = CCW;  %HERE -1 MEANS CCW


fprintf('Whole heart summary statistics:\n')
fprintf('Area: %d pixels\n', whole_heart_stats(1,1));
fprintf('Angle of AVJ to outflow: %f degrees (0-360)', whole_heart_stats(1,2));
if whole_heart_stats(1,2) == -1;
    fprintf('COUNTER-CLOCKWISE\n');
else
    fprintf('CLOCKWISE\n');
end;

fprintf('Click on image to continue\n');
ginput(1);
fig = gcf;
close(fig);

end